from pathlib import Path

from integra_bridge.adapters.base_adapter import BaseAdapter

from integra_bridge.common.dependency_manager import dm
from integra_bridge.dto.responces.external_service import ExternalServiceConfigResponse


class ConfigurationHandler:

    @classmethod
    async def get_configurations(cls):

        from integra_bridge.common.enums import AdapterType
        processors = BaseAdapter.get_adapters(adapter_type=AdapterType.processors)
        processor_views = []
        for processor in processors:
            processor_view = await processor.get_view()
            processor_views.append(processor_view)

        connectors = BaseAdapter.get_adapters(adapter_type=AdapterType.connectors)
        connector_views = []
        for connector in connectors:
            connector_view = await connector.get_view()
            connector_views.append(connector_view)

        manual_path = dm.manual_path.name if isinstance(dm.manual_path, Path) else dm.manual_path

        response = ExternalServiceConfigResponse(
            service_name=dm.title,
            service_address=dm.address,
            application_start_date=dm.application_start_date,
            processor_views=processor_views,
            connector_views=connector_views,
            manual_file_name=manual_path
        )
        return response



import json
from json import JSONDecodeError
from pathlib import Path

from pydantic import PositiveInt, StrictStr
from pydantic_settings import BaseSettings, SettingsConfigDict


class Settings(BaseSettings):

    def __init__(self):
        super().__init__()
        self.__serialize_url_map()

    model_config = SettingsConfigDict(
        case_sensitive=True,
        env_file=Path(__file__).parent.parent.parent / ".env",
        env_file_encoding="utf-8",
        extra="allow",
    )
    DEFAULT_CONNECTOR_TIMEOUT: PositiveInt | None = 120
    API_PREFIX: StrictStr = "/api/integra"
    URL_PATH_MAPPER: StrictStr | None = ""

    def __serialize_url_map(self):
        if self.URL_PATH_MAPPER:
            try:
                self.URL_PATH_MAPPER = json.loads(self.URL_PATH_MAPPER)
            except JSONDecodeError:
                self.URL_PATH_MAPPER = {}
        else:
            self.URL_PATH_MAPPER = {}


SETTINGS = Settings()



async def parse_string_path(input_string: str) -> dict:
    target_path_config = {
        'root': None,
        'sequention': []
    }
    if input_string.startswith('#body'):
        target_path_config['root'] = 'body'

    input_string = input_string.replace('#body.', '')
    if not input_string == '':
        target_path_config['sequention'] = input_string.split('.')
    return target_path_config


async def get_root_object_by_path(
        body: dict | list,
        path: dict,
        default_object: dict | list | None = None,
        use_sequention: bool = True,
        *args,
        **kwargs
) -> dict | list:
    if path['root'] == 'body':
        root_object = body
    # elif path['root'] == 'context':
    # todo: current = context
    elif not path['root'] and default_object:
        root_object = default_object
    # По умолчанию если нет других данных используем корень тела сообщения
    elif not path['root'] and not default_object:
        root_object = body
    else:
        raise KeyError(f'Invalid path: Have to use tag or give a current object')

    if not path['sequention']:
        return root_object

    if use_sequention and isinstance(root_object, dict) and path['sequention']:
        for elem in path['sequention']:
            root_object = root_object.setdefault(elem, {})
    return root_object


async def set_by_sequention(meta: dict, path: list[str], value, return_only_target: bool = False,
                            default_filed_name: str = 'set_by_sequention') -> dict | list:
    """
    Устанавливает значение value в словаре data по списку ключей path.
    Если каких-то ключей по пути нет, они будут созданы.
    Возвращает исходный словарь data (изменённый).
    """
    if not path:
        meta[default_filed_name] = value
        return meta
    current = meta
    for key in path[:-1]:
        current = current.setdefault(key, {})
    current[path[-1]] = value
    return current[path[-1]] if return_only_target else meta



from abc import abstractmethod, ABC

from integra_bridge.adapters.base_adapter import BaseAdapter
from integra_bridge.dto.responces.validation import ValidationResponse
from integra_bridge.entity.processed_components import ProcessedComponents
from integra_bridge.entity.processor import Processor
from integra_bridge.common.enums import AdapterType


class ProcessorAdapter(BaseAdapter, ABC):

    def __init__(self):
        super().__init__()
        self.__view = None
        BaseAdapter.add_adapter(self, adapter_type=AdapterType.processors)

    def __del__(self):
        BaseAdapter.remove_adapter(self, adapter_type=AdapterType.processors)

    @abstractmethod
    async def execute(self, components: ProcessedComponents) -> ProcessedComponents:
        ...

    async def validate(self, processor: Processor) -> ValidationResponse:
        return ValidationResponse(result=True)
